# netmap - Perl/GTK Network Scanner
#
# Copyright (C) 2003 Joshua Corbin <jcorbin@jdweb.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package netmap::consoleDialog;
use strict;
use Gtk;
use netmap::console;
our @ISA = ('netmap::console');

sub new {
  my $proto = shift || return undef;
  my %param  = @_;
  my $class = ref $proto || $proto;
  my $self  = $class->SUPER::new(%param) || return undef;
  bless $self, $class;
  $self->{_delcback} = $param{delete_callback};
  return $self;
}

sub show {
  my $self = shift || return undef;
  my %param = @_;
  $self->{win}->position($param{position}) if (defined $self->{position});
  $self->{win}->show_all;
}

sub _delete {
  my $self = shift || return undef;
  if (ref $self->{_delcback} eq 'ARRAY') {
    my @c = @{$self->{_delcback}};
    my $code = shift @c;
    if (ref $code eq 'CODE') {
      &$code($self, @c);
    } else {
      return undef;
    }
  } else {
    return undef;
  }
  return 1;
}

sub _build {
  my $self = shift || return undef;
  my %param = @_;
  $self->SUPER::_build(%param) || return undef;
  $self->{title}  = $param{title}  || '<CHANGE ME>';
  $self->{wid}    = $param{width}  || 400;
  $self->{hei}    = $param{height} || 300;
  $self->{parent} = $param{parentWindow};
  delete $param{title};
  delete $param{width};
  delete $param{height};
  delete $param{parentWindow};
  my $win  = new Gtk::Dialog;
  my $close = new Gtk::Button('Close');
  my $teardown =
    sub {
      $self->_nuke;
      $self->_delete;
      $win->destroy;
    };
  $close->signal_connect('clicked',    $teardown);
  $win->signal_connect('delete_event', $teardown);
  $win->action_area->add($close);
  $win->vbox->pack_start($self->{_widget}, 1, 1, 0);
  $win->set_title($self->{title});
  $win->set_default_size($self->{wid}, $self->{hei});
  $win->set_transient_for($self->{parent}) if (defined $self->{parent});
  $self->{win}   = $self->{_widget} = $win;
  $self->{close} = $close;
  return 1;
}

1;
