# netmap - Perl/GTK Network Scanner
#
# Copyright (C) 2003 Joshua Corbin <jcorbin@jdweb.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package netmap::ctreeHost;
use strict;
use netmap::host;
use Data::Dumper;

our $AUTOLOAD;
our @ISA = ('netmap::host');

# new( Gtk::CTree ctree, [Gtk::CTreeNode parent] )
sub new {
  my $proto  = shift || return undef;
  my $ctree  = shift || return undef;
  my $parent = shift;
  (ref $ctree eq 'Gtk::CTree') || return undef;
  my $class = ref $proto || $proto;
  my $self = $class->SUPER::new(@_) || return undef;
  bless $self, $class;
  $self->{ctree} = $ctree;
  $self->{subnetclass} = 'netmap::ctreeSubnet';
  my %param = @_;
  if (defined $param{fromser}) {
    $self->deserialize($param{fromser});
  } elsif (defined $param{deserdata}) {
    $self->deserial_data($param{deserdata});
  }
  $self->_create($parent) || return undef;
  return $self;
}

sub _create {
  my $self   = shift || return undef;
  my $parent = shift;
  my $wid    = $parent->{_widget};# if(ref $parent eq $self->{subnetclass});
  my $title  = $self->ipv4_address;
  return undef if ((defined $wid) && (ref $wid ne 'Gtk::CTreeNode'));
  $parent->add($self) || return undef;
  $self->{_widget} = $self->{ctree}->insert
    ($wid, undef, [$title], 1, undef, undef, undef, undef, 1, 0) || return undef;
  return 1;
}

sub AUTOLOAD {
  my $self  = shift || return undef;
  my @a     = split '::', $AUTOLOAD;
  my $field = $a[-1];
  my $value = shift;
  my $ret   = $self->SUPER::__field($field, $value);
  if ((($field eq 'ipv4_address') || ($field eq 'hostname')) && (defined $value) &&
      (defined $self->{ctree}) && (defined $self->{_widget})) {
    my $title = $self->SUPER::ipv4_address;
    $self->{ctree}->node_set_text($self->{_widget}, 0, $title);
  }
  return $ret;
}

1;

# sub indexof {
#   my $it = shift || return undef;
#   my $i;
#   for ($i=0;$i<=$#_;$i++) {
#     return $i if ($_[$i] eq $it);
#   }
#   return undef;
# }
