# netmap - Perl/GTK Network Scanner
#
# Copyright (C) 2003 Joshua Corbin <jcorbin@jdweb.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package netmap::ctreeSubnet;
use strict;
use netmap::subnet;
use netmap::ctreeHost;

our $AUTOLOAD;
our @ISA = ('netmap::subnet');

# new( Gtk::CTree ctree, [Gtk::CTreeNode parent] )
sub new {
  my $proto  = shift || return undef;
  my $ctree  = shift || return undef;
  my $parent = shift;
  (ref $ctree eq 'Gtk::CTree') || return undef;
  my $class = ref $proto || $proto;
  my $self  = $class->SUPER::new(@_);
  bless $self, $class;
  my %param = @_;
  $self->{ctree}       = $ctree;
  $self->{hostclass}   = 'netmap::ctreeHost';
  $self->{subnetclass} = 'netmap::ctreeSubnet';
  if (defined $param{fromser}) {
    $self->deserialize($param{fromser});
  } elsif (defined $param{deserdata}) {
    $self->deserial_data($param{deserdata});
  }
  $self->_create($parent) || return undef;
  return $self;
}

sub del {
  my $self = shift || return undef;
  my $this = shift || return undef;
  my $obj;
  if (!ref $this) {
    ($obj) = map {$_->[1]} grep {$_->[0] eq $this} @{$self->{children}};
  } else {
    $obj = $this;
    return undef if ((ref $obj ne $self->{hostclass}) && (ref $obj ne $self->{subnetclass}));
  }
  $self->{ctree}->remove($obj->{_widget}) if (defined $obj->{_widget});
  $self->SUPER::del($this);
}

sub _rebuild_subnet {
  my $self = shift || return undef;
  my $data = shift || return undef;
  if (defined $self->{_widget}) {
    my $subnet = new netmap::ctreeSubnet($self->{ctree}, $self, deserdata => $data);
  } else {
    push @{$self->{delaysub}}, $data;
  }
}

sub _rebuild_host {
  my $self = shift || return undef;
  my $data = shift || return undef;
  if (defined $self->{_widget}) {
    my $host = new netmap::ctreeHost($self->{ctree}, $self, deserdata => $data);
  } else {
    push @{$self->{delayhost}}, $data;
  }
}

sub _create {
  my $self   = shift || return undef;
  my $parent = shift;
  my $wid    = $parent->{_widget} if(ref $parent eq $self->{subnetclass});
  $parent->add($self) || return undef;
  $self->{_widget} = $self->{ctree}->insert
    ($wid, undef, [$self->number], 1, undef, undef, undef, undef, 0, 1);
  local $_;
  if (ref $self->{delaysub} eq 'ARRAY') {
    for (@{$self->{delaysub}}) {
      netmap::ctreeSubnet->new($self->{ctree}, $self, deserdata => $_);
    }
    delete $self->{delaysub};
  }
  if (ref $self->{delayhost} eq 'ARRAY') {
    for (@{$self->{delayhost}}) {
      netmap::ctreeHost->new($self->{ctree}, $self, deserdata => $_);
    }
    delete $self->{delayhost};
  }
  return 1;
}

sub AUTOLOAD {
  my $self  = shift || return undef;
  my @a     = split '::', $AUTOLOAD;
  my $field = $a[-1];
  my $value = shift;
  my $ret   = $self->SUPER::__field($field, $value);
  if (($field eq 'number') && (defined $value) &&
      (defined $self->{ctree}) && (defined $self->{_widget})) {
    $self->{ctree}->node_set_text($self->{_widget}, 0, $self->SUPER::number);
  }
  return $ret;
}

1;
