# netmap - Perl/GTK Network Scanner
#
# Copyright (C) 2003 Joshua Corbin <jcorbin@jdweb.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package netmap::nmap::clist;
use strict;
use Gtk;


sub new {
  my $proto = shift || return undef;
  my $class = ref $proto || $proto;
  my $nmap  = shift || return undef;
  my $self  = {};
  bless $self, $class;
  my %param = @_;
  $self->{nmap}   = $nmap;
  $self->{mode}   = $param{mode} || 'multiple';
#   $self->{tfield} = $param{tfield};
  $self->{scroll} = new Gtk::ScrolledWindow(undef, undef);
  $self->{list}   = Gtk::CList->new_with_titles($param{title});
  $self->{list}->set_selection_mode(3) if ($self->{mode} eq 'multiple');
  $self->{scroll}->add_with_viewport($self->{list});
  $self->{scroll}->set_policy(1,1);
  return $self;
}

sub _setup {
  my $self = shift || return undef;

  $self->updateui || return undef;

  $self->{list}->signal_connect('select-row',   sub {$self->update});
  $self->{list}->signal_connect('unselect-row', sub {$self->update})
    if($self->{mode} eq 'multiple');

#   my $upd =
#     sub {
#       if(defined $self->{nmap}->{$self->{tfield}}) {
# 	$self->updateui;
# 	delete $self->{nmap}->{$self->{tfield}};
#       }
#     };
#   $self->{ttag} = Gtk->timeout_add(500, $upd) if (defined $self->{tfield});

  return 1;
}

# sub DESTROY {
#   my $self = shift || return undef;
#   Gtk->timeout_remove($self->{ttag});
# }

sub updateui {
  my $self = shift || return undef;
  $self->{_lockui} = 1;
  $self->{list}->clear;
  map {$self->{list}->append($_)} sort values %{$self->{types}};
  if ($self->{mode} eq 'multiple') {
    for (my $i=0; $i<$self->{list}->rows; $i++) {
      if (isElemOf(keybyvalue($self->{list}->get_text($i,0), \%{$self->{types}}),
		   @{$self->{array}})) {
	$self->{list}->select_row($i,0);
      }
    }
  } elsif ($self->{mode} eq 'single') {
    for (my $i=0; $i<$self->{list}->rows; $i++) {
      if ($self->{list}->get_text($i,0) eq
	  $self->{types}->{$self->{array}->[0]}) {
	$self->{list}->select_row($i, 0);
	last;
      }
    }
  }
  delete $self->{_lockui};
  return 1;
}

sub update {
  my $self = shift || return undef;
  return 1 if (defined $self->{_lockui});
  if ($self->{mode} eq 'multiple') {
    @{$self->{array}} =
      map {keybyvalue($_, \%{$self->{types}})}
	map {$self->{list}->get_text($_,0)}
	  $self->{list}->selection;
  } elsif ($self->{mode} eq 'single') {
    @{$self->{array}} = (keybyvalue($self->{list}->get_text($self->{list}->selection, 0), $self->{types}));
  }
  return 1;
}

sub isElemOf {
  my $it = shift || return undef;
  local $_;
  for (@_) {
    return 1 if($_ eq $it);
  }
  return 0;
}

sub keybyvalue {
  my ($v, $h) = @_;
  local $_;
  for my $k (keys %$h) {
    return $k if($h->{$k} eq $v);
  }
  return undef;
}

1;
