# netmap - Perl/GTK Network Scanner
#
# Copyright (C) 2003 Joshua Corbin <jcorbin@jdweb.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package netmap::nmap::decoys;
use strict;
use Gtk;

sub new {
  my $proto = shift || return undef;
  my $class = ref $proto || $proto;
  my $nmap  = shift || return undef;
  my $self  = {nmap => $nmap};
  bless $self, $class;

  $self->{lay} = new Gtk::VBox(0);
  $self->_build || return undef;

  return $self->{lay};
}

sub _build {
  my $self = shift || return undef;
  $self->{list} = Gtk::CList->new_with_titles('Decoys');
  $self->{lay}->pack_start($self->{list}, 1, 1, 0);
  my $bar  = new Gtk::HBox(0);
  my $del  = new Gtk::Button('Del');
  my $text = new Gtk::Entry;
  my $add  = new Gtk::Button('Add');
  my $use  = new Gtk::CheckButton('Use Decoys');
  $use->set_active($self->{nmap}->{usedecoys});
  $bar->pack_start($text, 1, 1, 0);
  $bar->pack_start($add,  0, 0, 0);
  $bar->pack_start($del,  0, 0, 0);
  $self->{lay}->pack_start($bar, 0, 0, 0);
  $self->{lay}->pack_start($use, 0, 0, 0);
  $self->{text}   = $text;
  $self->{addupd} = $add;
  $self->{use}    = $use;
  map {$self->{list}->append($_)} @{$self->{nmap}->{decoys}};

  $self->{list}->signal_connect('select-row', sub {$self->selrow});
  $self->{list}->signal_connect('unselect-row', sub {$self->unselrow});
  $add->signal_connect('clicked', sub {$self->add});
  $del->signal_connect('clicked', sub {$self->del});
  $use->signal_connect('clicked', sub {$self->uset});

  return 1;
}

sub uset {
  my $self = shift || return undef;
  $self->{nmap}->{usedecoys} = $self->{use}->get_active;
}

sub selrow {
  my $self = shift || return undef;
  my $sel  = $self->{list}->selection;
  my $text = $self->{list}->get_text($sel, 0);
  $self->{cursel} = $sel;
  $self->{text}->set_text($text);
  $self->{addupd}->label('Update');
}

sub unselrow {
  my $self = shift || return undef;
  delete $self->{cursel};
  $self->{text}->set_text('');
  $self->{addupd}->label('Add');
}

sub add {
  my $self = shift || return undef;
  my $text = $self->{text}->get_text;
  if (defined $self->{cursel}) {
    my $old = $self->{list}->get_text($self->{cursel}, 0);
    $self->{list}->set_text($self->{cursel}, 0, $text);
    my @a = @{$self->{nmap}->{decoys}};
    @{$self->{nmap}->{decoys}} = ();
    local $_;
    for (@a) {
      if ($_ eq $old) {
	push @{$self->{nmap}->{decoys}}, $text;
      } else {
	push @{$self->{nmap}->{decoys}}, $_;
      }
    }
    $self->{text}->set_text('');
    $self->{addupd}->label('Add');
    $self->{list}->unselect_all;
    delete $self->{cursel};
  } else {
    $self->{list}->append($text);
    push @{$self->{nmap}->{decoys}}, $text;
    $self->{text}->set_text('');
  }
}

sub del {
  my $self = shift || return undef;
  my $sel  = $self->{list}->selection;
  my $text = $self->{list}->get_text($sel, 0);
  $self->{list}->remove($sel);
  @{$self->{nmap}->{decoys}} = grep {$_ ne $text} @{$self->{nmap}->{decoys}};
}

1;
