# netmap - Perl/GTK Network Scanner
#
# Copyright (C) 2003 Joshua Corbin <jcorbin@jdweb.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package netmap::nmap::nmapConsole;
use strict;
use Gtk;
use netmap::console;

sub new {
  my $proto = shift || return undef;
  my $class = ref $proto || $proto;
  my %param = @_;
  return if (ref $param{nmap} ne 'netmap::nmap');
  my $self =
    {nmap    => $param{nmap},
     stopsig => $param{stopsig} || 15};
  bless $self, $class;

  $self->_build(%param) || return undef;

  return $self;
}

sub widget {
  my $self = shift || return undef;
  return $self->{lay};
}

sub setcont {
  my $self = shift || return undef;
  my $wid  = shift || return undef;
  $self->clrcont   || return undef;
  $self->{cont}->add($wid);
  $wid->show_all;
  $self->{curcld} = $wid;
  return 1;
}

sub clrcont {
  my $self = shift || return undef;
  if (defined $self->{curcld}) {
    $self->{cont}->remove($self->{curcld});
    delete $self->{curcld};
  }
  return 1;
}

sub addtoqueue {
  my $self = shift || return undef;
  my %param = @_;
  return undef unless (defined $param{title});
  my $dup = 0;
  for (my $i=0; $i<$self->{queue}->rows; $i++) {
    if ($self->{queue}->get_text($i, 0) eq $param{title}) {
      $dup = 1;
      last;
    }
  }
  return undef if ($dup);
  $self->{queue}->append($param{title});
  return 1;
}

sub delfromqueue {
  my $self  = shift || return undef;
  my $title = shift || return undef;
  for (my $i=0; $i<$self->{queue}->rows; $i++) {
    if ($self->{queue}->get_text($i, 0) eq $title) {
      $self->{queue}->remove($i);
      last;
    }
  }
}

sub addscanner {
  my $self = shift || return undef;
  my %param = @_;
  return if ((!defined $param{pid}) || (!defined $param{pipe}) || (!defined $param{title}));
  return unless (kill 0, $param{pid});
  my ($title, $pipe, $pid) = ($param{title}, $param{pipe}, $param{pid});
  return undef if (defined $self->{_console}->{$title});
  $self->{_console}->{$title} = new netmap::console(app => $self->{nmap}->{app}) || return undef;
  $self->{_pid}->{$title}     = $pid;

  my $cback = $param{cback};
  my $done = sub {
    $self->{list}->unselect_all if ($self->{_cur} eq $title);
    for (my $i=0; $i<$self->{list}->rows; $i++) {
      if ($self->{list}->get_text($i, 0) eq $title) {
	$self->{list}->remove($i);
	last;
      }
    }
    if ($self->{_cur} eq $title) {
      $self->{clrcont};
      delete $self->{_cur};
    }
    delete $self->{_console}->{$title};
    delete $self->{_pid}->{$title};
    if (ref $cback eq 'CODE') {
      &$cback($self, $title);
    } elsif (ref $cback eq 'ARRAY') {
      my @c = @$cback;
      my $code = shift @c;
      if (ref $code eq 'CODE') {
	&$code(@c);
      }
    }
  };

  $self->{_console}->{$title}->attach($pipe, cback => $done);
  $self->{list}->append($param{title});
  for (my $i=0; $i<$self->{list}->rows; $i++) {
    if ($self->{list}->get_text($i, 0) eq $title) {
      $self->{list}->select_row($i, 0);
      last;
    }
  }
  return 1;
}

sub concur {
  my $self = shift || return undef;
  my $arg  = shift;
  if (defined $arg) {
    $self->{concur}->set_value($arg);
  } else {
    return $self->{concur}->get_value_as_int;
  }
}

sub _build {
  my $self = shift || return undef;
  my %param = @_;
  $self->{lay}     = new Gtk::VPaned;
  $self->{scroll1} = new Gtk::ScrolledWindow(undef, undef);
  $self->{scroll2} = new Gtk::ScrolledWindow(undef, undef);
  $self->{list}    = Gtk::CList->new_with_titles('Current Scanners');
  $self->{queue}   = Gtk::CList->new_with_titles('Queued Scanners');
  $self->{stop}    = new Gtk::Button('Stop');
  $self->{cont}    = new Gtk::Frame('Scanner Output');
  # value, lower, upper, step_increment, page_increment, page_size
  my $adj = new Gtk::Adjustment(3, -1, 99, 1, 1, 1);
  $self->{concur} = new Gtk::SpinButton($adj, 1, 0);
  $self->{run}     = new Gtk::Button('Run');
  $self->{concur}->set_numeric(1);

  my $lay         = new Gtk::VBox(0);
  $lay->pack_start($self->{cont}, 1, 1, 0);
  $lay->pack_start($self->{stop}, 0, 1, 0);

  $self->{scroll1}->add_with_viewport($self->{list});
  $self->{scroll1}->set_policy(1,1);
  $self->{scroll2}->add_with_viewport($self->{queue});
  $self->{scroll2}->set_policy(1,1);
  $self->{stop}->set_sensitive(0);

  my $h1 = new Gtk::HBox(1);
  my $v1 = new Gtk::VBox(0);
  my $v2 = new Gtk::VBox(0);
  my $h  = new Gtk::HBox(0);

  $h->pack_start(new Gtk::Label('Use'), 0, 0, 0);
  $h->pack_start($self->{concur},  0, 0, 0);
  $h->pack_start(new Gtk::Label('concurrent scanners.'), 0, 0, 0);
  $v2->pack_start($self->{scroll1}, 1, 1, 0);
  $v2->pack_start($h,               0, 0, 0);

  $h1->pack_start($v2,              1, 1, 0);
  $h1->pack_start($v1,              1, 1, 0);
  $v1->pack_start($self->{scroll2}, 1, 1, 0);
  $v1->pack_start($self->{run},     0, 0, 0);
  $self->{lay}->add1($h1);
  $self->{lay}->add2($lay);
  $self->{lay}->set_position( int(($self->{nmap}->{app}->{defsize}->[1])/3) );

  $self->{queue}->signal_connect('select-row',   sub {$self->{run}->set_sensitive(1)});
  $self->{queue}->signal_connect('unselect-row', sub {$self->{run}->set_sensitive(0)});
  $self->{run}->set_sensitive(0);
  $self->{run}->signal_connect
    ('clicked', sub {
       if (defined $self->{runcback}) {
	 my $title = $self->{queue}->get_text($self->{queue}->selection, 0);
	 &{$self->{runcback}}($title);
	 $self->{queue}->select_row(0, 0) if($self->{queue}->rows);
       }
     });

  my $sel = sub {
    my $title = $self->{list}->get_text($self->{list}->selection, 0);
    $self->{_cur} = $title;
    $self->setcont($self->{_console}->{$title}->widget);
    $self->{stop}->set_sensitive(1);
  };
  my $unsel = sub {
    if (defined $self->{_cur}) {
      $self->clrcont;
      delete $self->{_cur};
    }
    $self->{stop}->set_sensitive(0);
  };
  my $stop = sub {
    if (defined $self->{_pid}->{$self->{_cur}}) {
      kill $self->{stopsig}, $self->{_pid}->{$self->{_cur}} ||
	warn "kill: $!\n";
    }
  };
  $self->{list}->signal_connect('select-row',   $sel);
  $self->{list}->signal_connect('unselect-row', $unsel);
  $self->{stop}->signal_connect('clicked',      $stop);
  $self->{concur}->signal_connect('changed', sub {$self->{scanner}->doqueue});
  return 1;
}

1;
