# netmap - Perl/GTK Network Scanner
#
# Copyright (C) 2003 Joshua Corbin <jcorbin@jdweb.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package netmap::nmap::parser;
use strict;
use XML::Simple;

sub parsefile {
  my $file = shift;
  my %param = @_;
  my $mode = $param{mode} || 'host';
  local $/;
  my $in;
  unless (open $in, $file) {
    warn "Couldn't open $file: $!\n";
    return undef;
  }
  my $xml = <$in>;
  close $in;
  my $data = eval {XMLin($xml, keyattr => undef, forcearray => ['host', 'hostnames', 'port', 'osmatch'])};
  if (!defined $data) {
    warn "Error reading XML from $file\n";
    return undef;
  }
  my @ret;
  for my $host (grep {lc $_->{status}->{state} eq 'up'} @{$data->{host}}) {
    @{$host->{ports}->{port}} =
      map {my $a = {};
	   $a->{number}   = $_->{portid};
	   $a->{state}    = $_->{state}->{state};
	   $a->{protocol} = $_->{protocol};
	   $a->{service}  = $_->{service}->{name} if(defined $_->{service}->{name});
	   $a->{owner}    = $_->{owner}->{name}   if(defined $_->{owner}->{name});
	   [$a->{protocol}, $a->{number}, $a->{service} || undef]}
      grep {lc $_->{state}->{state} eq 'open'}
      @{$host->{ports}->{port}}
	  if (ref $host->{ports}->{port} eq 'ARRAY');
    delete $host->{status};
    @{$host->{hostnames}} = map {$_->{hostname}->{name}} @{$host->{hostnames}};
    $host->{ipv4_address} = $host->{address}->{addr} if(lc $host->{address}->{addrtype} eq 'ipv4');
    delete $host->{address};
    $host->{os} = [map {[$_->{accuracy}, $_->{name}]} @{$host->{os}->{osmatch}}];
    $host->{ports} = $host->{ports}->{port};
    push @ret, $host;
  }
  if ($mode eq 'subnet') {
    @ret = map {$_->{ipv4_address}} @ret;
  }
  return [@ret];
}

1;
